// Use LANG environment variable to choose locale
pref("intl.locale.matchOS", true);

// Disable default mailer checking.
pref("mail.shell.checkDefaultMail", false);

// Enable Network Manager integration
pref("network.manage-offline-status", true);

// identify default locale to use for searchplugins
pref("distribution.searchplugins.defaultLocale", "en-US");

// Don't disable our bundled extensions in the application directory
pref("extensions.autoDisableScopes", 0);
pref("extensions.shownSelectionUI", true);

// Map to hyphenation patterns from openoffice.org-hyphenation and openoffice.org-dictionaries
pref("intl.hyphenation-alias.af", "af-za");
pref("intl.hyphenation-alias.af-*", "af-za");
pref("intl.hyphenation-alias.bn", "bn-in");
pref("intl.hyphenation-alias.bn-*", "bn-in");
pref("intl.hyphenation-alias.ca-*", "ca");
pref("intl.hyphenation-alias.cs", "cs-cz");
pref("intl.hyphenation-alias.cs-*", "cs-cz");
pref("intl.hyphenation-alias.da", "da-dk");
pref("intl.hyphenation-alias.da-*", "da-dk");
pref("intl.hyphenation-alias.de", "de-de");
pref("intl.hyphenation-alias.de-*", "de-de");
pref("intl.hyphenation-alias.de-AT-1901", "de-de");
pref("intl.hyphenation-alias.de-CH-*", "de-de");
pref("intl.hyphenation-alias.de-DE-1901", "de-de");
pref("intl.hyphenation-alias.el", "el-gr");
pref("intl.hyphenation-alias.el-*", "el-gr");
pref("intl.hyphenation-alias.en", "en-us");
pref("intl.hyphenation-alias.en-*", "en-us");
pref("intl.hyphenation-alias.es", "es-es");
pref("intl.hyphenation-alias.es-*", "es-es");
pref("intl.hyphenation-alias.et", "et-ee");
pref("intl.hyphenation-alias.et-*", "et-ee");
pref("intl.hyphenation-alias.fi", "fi-fi");
pref("intl.hyphenation-alias.fi-*", "fi-fi");
pref("intl.hyphenation-alias.fr-*", "fr");
pref("intl.hyphenation-alias.ga", "ga-ie");
pref("intl.hyphenation-alias.ga-*", "ga-ie");
pref("intl.hyphenation-alias.gu", "gu-in");
pref("intl.hyphenation-alias.gu-*", "gu-in");
pref("intl.hyphenation-alias.hi", "hi-in");
pref("intl-hyphenation-alias.hi-in", "hi-in");
pref("intl.hyphenation-alias.hr", "hr-hr");
pref("intl.hyphenation-alias.hr-*", "hr-hr");
pref("intl.hyphenation-alias.hu", "hu-hu");
pref("intl.hyphenation-alias.hu-*", "hu-hu");
pref("intl.hyphenation-alias.id", "id-id");
pref("intl-hyphenation-alias.id-*", "id-id");
pref("intl.hyphenation-alias.is", "is-is");
pref("intl.hyphenation-alias.is-*", "is-is");
pref("intl.hyphenation-alias.it", "it-it");
pref("intl.hyphenation-alias.it-*", "it-it");
pref("intl.hyphenation-alias.kn", "kn-in");
pref("intl.hyphenation-alias.kn-*", "kn-in");
pref("intl.hyphenation-alias.lt", "lt-lt");
pref("intl.hyphenation-alias.lt-*", "lt-lt");
pref("intl.hyphenation-alias.lv", "lv-lv");
pref("intl.hyphenation-alias.lv-*", "lv-lv");
pref("intl.hyphenation-alias.nb", "nb-no");
pref("intl.hyphenation-alias.nb-*", "nb-no");
pref("intl.hyphenation-alias.nl", "nl-nl");
pref("intl.hyphenation-alias.nl-*", "nl-nl");
pref("intl.hyphenation-alias.nn", "nn-no");
pref("intl.hyphenation-alias.nn-*", "nn-no");
pref("intl.hyphenation-alias.pa", "pa-in");
pref("intl.hyphenation-alias.pa-*", "pa-in");
pref("intl.hyphenation-alias.pl", "pl-pl");
pref("intl.hyphenation-alias.pl-*", "pl-pl");
pref("intl.hyphenation-alias.pt", "pt-pt");
pref("intl.hyphenation-alias.pt-*", "pt-pt");
pref("intl.hyphenation-alias.ro", "ro-ro");
pref("intl.hyphenation-alias.ro-*", "ro-ro");
pref("intl.hyphenation-alias.ru", "ru-ru");
pref("intl.hyphenation-alias.ru-*", "ru-ru");
pref("intl.hyphenation-alias.sh-*", "sh");
pref("intl.hyphenation-alias.sk", "sk-sk");
pref("intl.hyphenation-alias.sk-*", "sk-sk");
pref("intl.hyphenation-alias.sl", "sl-si");
pref("intl.hyphenation-alias.sl-*", "sl-si");
pref("intl.hyphenation-alias.sr-*", "sr");
pref("intl.hyphenation-alias.sv", "sv-se");
pref("intl.hyphenation-alias.sv-*", "sv-se");
pref("intl.hyphenation-alias.uk", "uk-ua");
pref("intl.hyphenation-alias.uk-*", "uk-ua");
pref("intl.hyphenation-alias.zu", "zu-za");
pref("intl.hyphenation-alias.zu-*", "zu-za");

